@@ -81,17 +81,30 @@ thread_local! {
 			("objectFieldsEx".into(), builtin_object_fields_ex),
 			("objectHasEx".into(), builtin_object_has_ex),
 			("slice".into(), builtin_slice),
+			("substr".into(), builtin_substr),
 			("primitiveEquals".into(), builtin_primitive_equals),
 			("equals".into(), builtin_equals),
 			("modulo".into(), builtin_modulo),
 			("mod".into(), builtin_mod),
 			("floor".into(), builtin_floor),
+			("ceil".into(), builtin_ceil),
 			("log".into(), builtin_log),
 			("pow".into(), builtin_pow),
+			("sqrt".into(), builtin_sqrt),
+			("sin".into(), builtin_sin),
+			("cos".into(), builtin_cos),
+			("tan".into(), builtin_tan),
+			("asin".into(), builtin_asin),
+			("acos".into(), builtin_acos),
+			("atan".into(), builtin_atan),
+			("exp".into(), builtin_exp),
+			("mantissa".into(), builtin_mantissa),
+			("exponent".into(), builtin_exponent),
 			("extVar".into(), builtin_ext_var),
 			("native".into(), builtin_native),
 			("filter".into(), builtin_filter),
 			("map".into(), builtin_map),
+			("flatMap".into(), builtin_flatmap),
 			("foldl".into(), builtin_foldl),
 			("foldr".into(), builtin_foldr),
 			("sortImpl".into(), builtin_sort_impl),
@@ -99,8 +112,11 @@ thread_local! {
 			("range".into(), builtin_range),
 			("char".into(), builtin_char),
 			("encodeUTF8".into(), builtin_encode_utf8),
+			("decodeUTF8".into(), builtin_decode_utf8),
 			("md5".into(), builtin_md5),
 			("base64".into(), builtin_base64),
+			("base64DecodeBytes".into(), builtin_base64_decode_bytes),
+			("base64Decode".into(), builtin_base64_decode),
 			("trace".into(), builtin_trace),
 			("join".into(), builtin_join),
 			("escapeStringJson".into(), builtin_escape_string_json),
@@ -108,6 +124,7 @@ thread_local! {
 			("reverse".into(), builtin_reverse),
 			("id".into(), builtin_id),
 			("strReplace".into(), builtin_str_replace),
+			("splitLimit".into(), builtin_splitlimit),
 			("parseJson".into(), builtin_parse_json),
 		].iter().cloned().collect()
 	};
@@ -229,6 +246,17 @@ fn builtin_slice(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc)
 	})
 }
 
+fn builtin_substr(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "substr", args, 3, [
+		0, str: ty!(string) => Val::Str;
+		1, from: ty!(BoundedNumber<(Some(0.0)), (None)>) => Val::Num;
+		2, len: ty!(BoundedNumber<(Some(0.0)), (None)>) => Val::Num;
+	], {
+		let out: String = str.chars().skip(from as usize).take(len as usize).collect();
+		Ok(Val::Str(out.into()))
+	})
+}
+
 fn builtin_primitive_equals(
 	context: Context,
 	_loc: Option<&ExprLocation>,
@@ -277,6 +305,14 @@ fn builtin_floor(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc)
 	})
 }
 
+fn builtin_ceil(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "ceil", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.ceil()))
+	})
+}
+
 fn builtin_log(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
 	parse_args!(context, "log", args, 1, [
 		0, n: ty!(number) => Val::Num;
@@ -294,6 +330,97 @@ fn builtin_pow(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -
 	})
 }
 
+fn builtin_sqrt(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "sqrt", args, 1, [
+		0, x: ty!(BoundedNumber<(Some(0.0)), (None)>) => Val::Num;
+	], {
+		Ok(Val::Num(x.sqrt()))
+	})
+}
+
+fn builtin_sin(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "sin", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.sin()))
+	})
+}
+
+fn builtin_cos(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "cos", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.cos()))
+	})
+}
+
+fn builtin_tan(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "tan", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.tan()))
+	})
+}
+
+fn builtin_asin(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "asin", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.asin()))
+	})
+}
+
+fn builtin_acos(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "acos", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.acos()))
+	})
+}
+
+fn builtin_atan(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "atan", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.atan()))
+	})
+}
+
+fn builtin_exp(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "exp", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(x.exp()))
+	})
+}
+
+fn frexp(s: f64) -> (f64, i16) {
+	if 0.0 == s {
+		(s, 0)
+	} else {
+		let lg = s.abs().log2();
+		let x = (lg - lg.floor() - 1.0).exp2();
+		let exp = lg.floor() + 1.0;
+		(s.signum() * x, exp as i16)
+	}
+}
+
+fn builtin_mantissa(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "mantissa", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(frexp(x).0))
+	})
+}
+
+fn builtin_exponent(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "exponent", args, 1, [
+		0, x: ty!(number) => Val::Num;
+	], {
+		Ok(Val::Num(frexp(x).1.into()))
+	})
+}
+
 fn builtin_ext_var(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
 	parse_args!(context, "extVar", args, 1, [
 		0, x: ty!(string) => Val::Str;
@@ -331,6 +458,40 @@ fn builtin_map(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -
 	})
 }
 
+fn builtin_flatmap(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
+	parse_args!(context, "flatMap", args, 2, [
+		0, func: ty!(function) => Val::Func;
+		1, arr: ty!((array | string));
+	], {
+		match arr {
+			Val::Str(s) => {
+				let mut out = String::new();
+				for c in s.chars() {
+					match func.evaluate_values(context.clone(), &[Val::Str(c.to_string().into())])? {
+						Val::Str(o) => out.push_str(&o),
+						_ => throw!(RuntimeError("in std.join all items should be strings".into())),
+					};
+				}
+				Ok(Val::Str(out.into()))
+			},
+			Val::Arr(a) => {
+				let mut out = Vec::new();
+				for el in a.iter() {
+					let el = el?;
+					match func.evaluate_values(context.clone(), &[el])? {
+						Val::Arr(o) => for oe in o.iter() {
+							out.push(oe?)
+						},
+						_ => throw!(RuntimeError("in std.join all items should be arrays".into())),
+					};
+				}
+				Ok(Val::Arr(out.into()))
+			},
+			_ => unreachable!(),
+		}
+	})
+}
+
 fn builtin_foldl(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
 	parse_args!(context, "foldl", args, 3, [
 		0, func: ty!(function) => Val::Func;
@@ -425,6 +586,23 @@ fn builtin_encode_utf8(
 	})
 }
 
+fn builtin_decode_utf8(
+	context: Context,
+	_loc: Option<&ExprLocation>,
+	args: &ArgsDesc,
+) -> Result<Val> {
+	parse_args!(context, "decodeUTF8", args, 1, [
+		0, arr: ty!((Array<ubyte>)) => Val::Arr;
+	], {
+		let data: Result<Vec<u8>> = arr.iter().map(|v| v.map(|v| match v{
+			Val::Num(n) => n as u8,
+			_ => unreachable!(),
+		})).collect();
+		let data = data?;
+		Ok(Val::Str(String::from_utf8(data).map_err(|_| RuntimeError("bad utf8".into()))?.into()))
+	})
+}
+
 fn builtin_md5(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
 	parse_args!(context, "md5", args, 1, [
 		0, str: ty!(string) => Val::Str;
@@ -468,6 +646,39 @@ fn builtin_base64(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc
 	})
 }
 
+fn builtin_base64_decode_bytes(
+	context: Context,
+	_loc: Option<&ExprLocation>,
+	args: &ArgsDesc,
+) -> Result<Val> {
+	parse_args!(context, "base64DecodeBytes", args, 1, [
+		0, input: ty!(string) => Val::Str;
+	], {
+		Ok(Val::Arr(
+			base64::decode(&input.as_bytes())
+				.map_err(|_| RuntimeError("bad base64".into()))?
+				.iter()
+				.map(|v| Val::Num(*v as f64)).collect::<Vec<_>>().into()
+		))
+	})
+}
+
+fn builtin_base64_decode(
+	context: Context,
+	_loc: Option<&ExprLocation>,
+	args: &ArgsDesc,
+) -> Result<Val> {
+	parse_args!(context, "base64Decode", args, 1, [
+		0, input: ty!(string) => Val::Str;
+	], {
+		Ok(Val::Str(
+			String::from_utf8(base64::decode(&input.as_bytes())
+				.map_err(|_| RuntimeError("bad base64".into()))?)
+				.map_err(|_| RuntimeError("bad utf8".into()))?.into()
+		))
+	})
+}
+
 fn builtin_join(context: Context, _loc: Option<&ExprLocation>, args: &ArgsDesc) -> Result<Val> {
 	parse_args!(context, "join", args, 2, [
 		0, sep: ty!((string | array));
@@ -594,6 +805,29 @@ fn builtin_str_replace(
 	})
 }
 
+fn builtin_splitlimit(
+	context: Context,
+	_loc: Option<&ExprLocation>,
+	args: &ArgsDesc,
+) -> Result<Val> {
+	parse_args!(context, "splitLimit", args, 3, [
+		0, str: ty!(string) => Val::Str;
+		1, c: ty!(char) => Val::Str;
+		2, maxsplits: ty!(number) => Val::Num;
+	], {
+		let maxsplits = maxsplits as isize;
+		let c = c.chars().next().unwrap();
+
+		let out: Vec<Val> = if maxsplits == -1 {
+			str.split(c).map(|s| Val::Str(s.into())).collect()
+		} else {
+			str.splitn(maxsplits as usize + 1, c).map(|s| Val::Str(s.into())).collect()
+		};
+
+		Ok(Val::Arr(out.into()))
+	})
+}
+
 pub fn call_builtin(
 	context: Context,
 	loc: Option<&ExprLocation>,
